/* SPDX-License-Identifier: GPL-2.0
 *
 * include/asm-sh/cpu-sh2a/freq.h
 *
 * Copyright (C) 2006  Yoshinori Sato
 */
#ifndef __ASM_CPU_SH2A_FREQ_H
#define __ASM_CPU_SH2A_FREQ_H

#define FREQCR	0xfffe0010

#endif /* __ASM_CPU_SH2A_FREQ_H */

