/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (c) 2010-2011, The Linux Foundation. All rights reserved.
 */

#ifndef _ASM_SUSPEND_H
#define _ASM_SUSPEND_H

static inline int arch_prepare_suspend(void)
{
	return 0;
}

#endif
