cmd_arch/x86/include/generated/uapi/asm/unistd_x32.h := sh '/build/linux-rOxhKL/linux-5.4.0/arch/x86/entry/syscalls/syscallhdr.sh' '/build/linux-rOxhKL/linux-5.4.0/arch/x86/entry/syscalls/syscall_64.tbl' 'arch/x86/include/generated/uapi/asm/unistd_x32.h' 'common,x32' '' '__X32_SYSCALL_BIT'
