cmd_arch/x86/include/generated/uapi/asm/unistd_64.h := sh '/build/linux-rOxhKL/linux-5.4.0/arch/x86/entry/syscalls/syscallhdr.sh' '/build/linux-rOxhKL/linux-5.4.0/arch/x86/entry/syscalls/syscall_64.tbl' 'arch/x86/include/generated/uapi/asm/unistd_64.h' 'common,64' '' ''
