/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright © 2014 NVIDIA Corporation
 * Copyright © 2015 Broadcom Corporation
 */

#ifndef __SOC_BRCMSTB_COMMON_H__
#define __SOC_BRCMSTB_COMMON_H__

bool soc_is_brcmstb(void);

#endif /* __SOC_BRCMSTB_COMMON_H__ */
