cmd_/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/fixdep-in.o :=  ld -r -o /build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/fixdep-in.o  /build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/fixdep.o
