cmd_/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/arch/x86/decode.o := gcc -Wp,-MD,/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/arch/x86/.decode.o.d -Wp,-MT,/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/arch/x86/decode.o -Werror -Wbad-function-cast -Wdeclaration-after-statement -Wformat-security -Wformat-y2k -Winit-self -Wmissing-declarations -Wmissing-prototypes -Wnested-externs -Wno-system-headers -Wold-style-definition -Wpacked -Wredundant-decls -Wstrict-prototypes -Wswitch-default -Wswitch-enum -Wundef -Wwrite-strings -Wformat -Wstrict-aliasing=3 -Wshadow -Wno-switch-default -Wno-switch-enum -Wno-packed -Wall -Wmissing-prototypes -Wstrict-prototypes -O2 -fomit-frame-pointer -std=gnu89 -g -I/build/linux-rOxhKL/linux-5.4.0/tools/include -I/build/linux-rOxhKL/linux-5.4.0/tools/arch/x86/include/uapi -I/build/linux-rOxhKL/linux-5.4.0/tools/arch/x86/include -I/build/linux-rOxhKL/linux-5.4.0/tools/lib -D"BUILD_STR(s)=$(pound)s" -I/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/arch/x86/lib -c -o /build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/arch/x86/decode.o arch/x86/decode.c

source_/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/arch/x86/decode.o := arch/x86/decode.c

deps_/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/arch/x86/decode.o := \
  /usr/include/stdc-predef.h \
  /usr/include/stdio.h \
  /usr/include/x86_64-linux-gnu/bits/libc-header-start.h \
  /usr/include/features.h \
  /usr/include/x86_64-linux-gnu/sys/cdefs.h \
  /usr/include/x86_64-linux-gnu/bits/wordsize.h \
  /usr/include/x86_64-linux-gnu/bits/long-double.h \
  /usr/include/x86_64-linux-gnu/gnu/stubs.h \
  /usr/include/x86_64-linux-gnu/gnu/stubs-64.h \
  /usr/lib/gcc/x86_64-linux-gnu/9/include/stddef.h \
  /usr/lib/gcc/x86_64-linux-gnu/9/include/stdarg.h \
  /usr/include/x86_64-linux-gnu/bits/types.h \
  /usr/include/x86_64-linux-gnu/bits/timesize.h \
  /usr/include/x86_64-linux-gnu/bits/typesizes.h \
  /usr/include/x86_64-linux-gnu/bits/time64.h \
  /usr/include/x86_64-linux-gnu/bits/types/__fpos_t.h \
  /usr/include/x86_64-linux-gnu/bits/types/__mbstate_t.h \
  /usr/include/x86_64-linux-gnu/bits/types/__fpos64_t.h \
  /usr/include/x86_64-linux-gnu/bits/types/__FILE.h \
  /usr/include/x86_64-linux-gnu/bits/types/FILE.h \
  /usr/include/x86_64-linux-gnu/bits/types/struct_FILE.h \
  /usr/include/x86_64-linux-gnu/bits/stdio_lim.h \
  /usr/include/x86_64-linux-gnu/bits/sys_errlist.h \
  /usr/include/x86_64-linux-gnu/bits/stdio.h \
  /usr/include/x86_64-linux-gnu/bits/stdio2.h \
  /usr/include/stdlib.h \
  /usr/include/x86_64-linux-gnu/bits/waitflags.h \
  /usr/include/x86_64-linux-gnu/bits/waitstatus.h \
  /usr/include/x86_64-linux-gnu/bits/floatn.h \
  /usr/include/x86_64-linux-gnu/bits/floatn-common.h \
  /usr/include/x86_64-linux-gnu/sys/types.h \
  /usr/include/x86_64-linux-gnu/bits/types/clock_t.h \
  /usr/include/x86_64-linux-gnu/bits/types/clockid_t.h \
  /usr/include/x86_64-linux-gnu/bits/types/time_t.h \
  /usr/include/x86_64-linux-gnu/bits/types/timer_t.h \
  /usr/include/x86_64-linux-gnu/bits/stdint-intn.h \
  /usr/include/endian.h \
  /usr/include/x86_64-linux-gnu/bits/endian.h \
  /usr/include/x86_64-linux-gnu/bits/endianness.h \
  /usr/include/x86_64-linux-gnu/bits/byteswap.h \
  /usr/include/x86_64-linux-gnu/bits/uintn-identity.h \
  /usr/include/x86_64-linux-gnu/sys/select.h \
  /usr/include/x86_64-linux-gnu/bits/select.h \
  /usr/include/x86_64-linux-gnu/bits/types/sigset_t.h \
  /usr/include/x86_64-linux-gnu/bits/types/__sigset_t.h \
  /usr/include/x86_64-linux-gnu/bits/types/struct_timeval.h \
  /usr/include/x86_64-linux-gnu/bits/types/struct_timespec.h \
  /usr/include/x86_64-linux-gnu/bits/select2.h \
  /usr/include/x86_64-linux-gnu/bits/pthreadtypes.h \
  /usr/include/x86_64-linux-gnu/bits/thread-shared-types.h \
  /usr/include/x86_64-linux-gnu/bits/pthreadtypes-arch.h \
  /usr/include/x86_64-linux-gnu/bits/struct_mutex.h \
  /usr/include/x86_64-linux-gnu/bits/struct_rwlock.h \
  /usr/include/alloca.h \
  /usr/include/x86_64-linux-gnu/bits/stdlib-bsearch.h \
  /usr/include/x86_64-linux-gnu/bits/stdlib-float.h \
  /usr/include/x86_64-linux-gnu/bits/stdlib.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/arch/x86/include/asm/insn.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/arch/x86/include/asm/inat.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/arch/x86/include/asm/inat_types.h \
  arch/x86/../../../arch/x86/lib/inat.c \
  arch/x86/../../../arch/x86/lib/../include/asm/insn.h \
  /build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/arch/x86/lib/inat-tables.c \
  arch/x86/../../../arch/x86/lib/insn.c \
  /usr/include/string.h \
  /usr/include/x86_64-linux-gnu/bits/types/locale_t.h \
  /usr/include/x86_64-linux-gnu/bits/types/__locale_t.h \
  /usr/include/strings.h \
  /usr/include/x86_64-linux-gnu/bits/strings_fortified.h \
  /usr/include/x86_64-linux-gnu/bits/string_fortified.h \
  arch/x86/../../../arch/x86/lib/../include/asm/inat.h \
  arch/x86/../../../arch/x86/lib/../include/asm/emulate_prefix.h \
  arch/x86/../../elf.h \
  /usr/include/gelf.h \
  /usr/include/libelf.h \
  /usr/lib/gcc/x86_64-linux-gnu/9/include/stdint.h \
  /usr/include/stdint.h \
  /usr/include/x86_64-linux-gnu/bits/wchar.h \
  /usr/include/x86_64-linux-gnu/bits/stdint-uintn.h \
  /usr/include/elf.h \
  /usr/include/x86_64-linux-gnu/bits/auxv.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/list.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/types.h \
  /usr/lib/gcc/x86_64-linux-gnu/9/include/stdbool.h \
  /usr/include/x86_64-linux-gnu/asm/types.h \
  /usr/include/asm-generic/types.h \
  /usr/include/asm-generic/int-ll64.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/arch/x86/include/uapi/asm/bitsperlong.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitsperlong.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/uapi/asm-generic/bitsperlong.h \
  /usr/include/x86_64-linux-gnu/asm/posix_types.h \
  /usr/include/x86_64-linux-gnu/asm/posix_types_64.h \
  /usr/include/asm-generic/posix_types.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/poison.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/kernel.h \
  /usr/include/assert.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/compiler.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/compiler-gcc.h \
  /usr/include/byteswap.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/hashtable.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/bitops.h \
  /usr/lib/gcc/x86_64-linux-gnu/9/include/limits.h \
  /usr/lib/gcc/x86_64-linux-gnu/9/include/syslimits.h \
  /usr/include/limits.h \
  /usr/include/x86_64-linux-gnu/bits/posix1_lim.h \
  /usr/include/x86_64-linux-gnu/bits/local_lim.h \
  /usr/include/linux/limits.h \
  /usr/include/x86_64-linux-gnu/bits/posix2_lim.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/bits.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/const.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/uapi/linux/const.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops/__ffs.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops/__ffz.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops/fls.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops/__fls.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops/fls64.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops/find.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops/hweight.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops/arch_hweight.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops/const_hweight.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops/atomic.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitops/non-atomic.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/hash.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/log2.h \
  arch/x86/../../arch.h \
  arch/x86/../../elf.h \
  arch/x86/../../cfi.h \
  arch/x86/../../warn.h \
  /usr/include/x86_64-linux-gnu/sys/stat.h \
  /usr/include/x86_64-linux-gnu/bits/stat.h \
  /usr/include/fcntl.h \
  /usr/include/x86_64-linux-gnu/bits/fcntl.h \
  /usr/include/x86_64-linux-gnu/bits/fcntl-linux.h \
  /usr/include/x86_64-linux-gnu/bits/fcntl2.h \

/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/arch/x86/decode.o: $(deps_/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/arch/x86/decode.o)

$(deps_/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/arch/x86/decode.o):
