/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __MACH_SDK7786_IRQ_H
#define __MACH_SDK7786_IRQ_H

/* arch/sh/boards/mach-sdk7786/irq.c */
extern void sdk7786_init_irq(void);

#endif /* __MACH_SDK7786_IRQ_H */
