/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
   md_u.h : user <=> kernel API between Linux raidtools and RAID drivers
          Copyright (C) 1998 Ingo Molnar
	  
*/
#ifndef _MD_U_H
#define _MD_U_H

#include <uapi/linux/raid/md_u.h>

extern int mdp_major;
#endif 
