/* SPDX-License-Identifier: GPL-2.0 */
// Copyright (C) 2005-2017 Andes Technology Corporation

#ifndef __NDS32_MMU_H
#define __NDS32_MMU_H

typedef struct {
	unsigned int id;
	void *vdso;
} mm_context_t;

#endif
