/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _DT_BINDINGS_SAMSUNG_I2S_H
#define _DT_BINDINGS_SAMSUNG_I2S_H

#define CLK_I2S_CDCLK		0
#define CLK_I2S_RCLK_SRC	1
#define CLK_I2S_RCLK_PSR	2

#endif /* _DT_BINDINGS_SAMSUNG_I2S_H */
