/* SPDX-License-Identifier: GPL-2.0 */
// Copyright (C) 2018 Hangzhou C-SKY Microsystems co.,ltd.

#ifndef __ASM_CSKY_FTRACE_H
#define __ASM_CSKY_FTRACE_H

#define MCOUNT_INSN_SIZE	14

#define HAVE_FUNCTION_GRAPH_FP_TEST

#define HAVE_FUNCTION_GRAPH_RET_ADDR_PTR

#define MCOUNT_ADDR	((unsigned long)_mcount)

#ifndef __ASSEMBLY__

extern void _mcount(unsigned long);

extern void ftrace_graph_call(void);

static inline unsigned long ftrace_call_adjust(unsigned long addr)
{
	return addr;
}

struct dyn_arch_ftrace {
};
#endif /* !__ASSEMBLY__ */
#endif /* __ASM_CSKY_FTRACE_H */
