cmd_arch/x86/include/generated/asm/syscalls_64.h := sh '/build/linux-rOxhKL/linux-5.4.0/arch/x86/entry/syscalls/syscalltbl.sh' /build/linux-rOxhKL/linux-5.4.0/arch/x86/entry/syscalls/syscall_64.tbl arch/x86/include/generated/asm/syscalls_64.h
