/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (c) 2010-2011, The Linux Foundation. All rights reserved.
 */

#ifndef _ASM_TLB_H
#define _ASM_TLB_H

#include <linux/pagemap.h>
#include <asm/tlbflush.h>

#include <asm-generic/tlb.h>

#endif
