/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (C) ST-Ericsson 2013
 * Author: Hongbo Zhang <hongbo.zhang@linaro.com>
 */

#ifndef PWR_AB8500_H
#define PWR_AB8500_H

extern const struct abx500_res_to_temp ab8500_temp_tbl_a_thermistor[];
extern const int ab8500_temp_tbl_a_size;

extern const struct abx500_res_to_temp ab8500_temp_tbl_b_thermistor[];
extern const int ab8500_temp_tbl_b_size;

#endif /* PWR_AB8500_H */
