/*
 * Architecture specific mm hooks
 */

#ifndef _ASM_GENERIC_MM_ARCH_HOOKS_H
#define _ASM_GENERIC_MM_ARCH_HOOKS_H

/*
 * This file should be included through arch/../include/asm/Kbuild for
 * the architecture which doesn't need specific mm hooks.
 *
 * In that case, the generic hooks defined in include/linux/mm-arch-hooks.h
 * are used.
 */

#endif /* _ASM_GENERIC_MM_ARCH_HOOKS_H */
