/* SPDX-License-Identifier: GPL-2.0-only */
/*
 *
 *  Copyright (C) 2011 Thomas Langer <thomas.langer@lantiq.com>
 */

#ifndef __FALCON_IRQ_H
#define __FALCON_IRQ_H

#include <falcon_irq.h>

#define NR_IRQS 328

#include_next <irq.h>

#endif
