/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __ASM_SH_CPU_SH2A_ADDRSPACE_H
#define __ASM_SH_CPU_SH2A_ADDRSPACE_H

#define P0SEG		0x00000000
#define P1SEG		0x00000000
#define P2SEG		0x20000000
#define P3SEG		0x40000000
#define P4SEG		0x60000000

#endif /* __ASM_SH_CPU_SH2A_ADDRSPACE_H */
