/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * GPIOs and interrupts for Voipac PXA270
 *
 * Copyright (C) 2010
 * Marek Vasut <marek.vasut@gmail.com>
 */

#ifndef _INCLUDE_VPAC270_H_
#define _INCLUDE_VPAC270_H_

#define	GPIO1_VPAC270_USER_BTN		1

#define	GPIO15_VPAC270_LED_ORANGE	15

#define	GPIO81_VPAC270_BKL_ON		81
#define	GPIO83_VPAC270_NL_ON		83

#define	GPIO52_VPAC270_SD_READONLY	52
#define	GPIO53_VPAC270_SD_DETECT_N	53

#define	GPIO84_VPAC270_PCMCIA_CD	84
#define	GPIO35_VPAC270_PCMCIA_RDY	35
#define	GPIO107_VPAC270_PCMCIA_PPEN	107
#define	GPIO11_VPAC270_PCMCIA_RESET	11
#define	GPIO17_VPAC270_CF_CD		17
#define	GPIO12_VPAC270_CF_RDY		12
#define	GPIO16_VPAC270_CF_RESET		16

#define	GPIO41_VPAC270_UDC_DETECT	41

#define	GPIO114_VPAC270_ETH_IRQ		114

#define	GPIO36_VPAC270_IDE_IRQ		36

#define	GPIO113_VPAC270_TS_IRQ		113

#endif
