/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * arch/arm/mach-iop32x/include/mach/irqs.h
 *
 * Author:	Rory Bolt <rorybolt@pacbell.net>
 * Copyright:	(C) 2002 Rory Bolt
 */

#ifndef __IRQS_H
#define __IRQS_H

#define NR_IRQS			33

#endif
