/* SPDX-License-Identifier: GPL-2.0-or-later */
#ifndef _ASM_POWERPC_DEBUGFS_H
#define _ASM_POWERPC_DEBUGFS_H

/*
 * Copyright 2017, Michael Ellerman, IBM Corporation.
 */

#include <linux/debugfs.h>

extern struct dentry *powerpc_debugfs_root;

#endif /* _ASM_POWERPC_DEBUGFS_H */
