cmd_/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/libctype.o := gcc -Wp,-MD,/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/.libctype.o.d -Wp,-MT,/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/libctype.o -Werror -Wbad-function-cast -Wdeclaration-after-statement -Wformat-security -Wformat-y2k -Winit-self -Wmissing-declarations -Wmissing-prototypes -Wnested-externs -Wno-system-headers -Wold-style-definition -Wpacked -Wredundant-decls -Wstrict-prototypes -Wswitch-default -Wswitch-enum -Wundef -Wwrite-strings -Wformat -Wstrict-aliasing=3 -Wshadow -Wno-switch-default -Wno-switch-enum -Wno-packed -Wall -Wmissing-prototypes -Wstrict-prototypes -O2 -fomit-frame-pointer -std=gnu89 -g -I/build/linux-rOxhKL/linux-5.4.0/tools/include -I/build/linux-rOxhKL/linux-5.4.0/tools/arch/x86/include/uapi -I/build/linux-rOxhKL/linux-5.4.0/tools/arch/x86/include -I/build/linux-rOxhKL/linux-5.4.0/tools/lib -D"BUILD_STR(s)=$(pound)s" -c -o /build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/libctype.o ../lib/ctype.c

source_/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/libctype.o := ../lib/ctype.c

deps_/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/libctype.o := \
  /usr/include/stdc-predef.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/ctype.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/compiler.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/compiler-gcc.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/linux/types.h \
  /usr/lib/gcc/x86_64-linux-gnu/9/include/stdbool.h \
  /usr/lib/gcc/x86_64-linux-gnu/9/include/stddef.h \
  /usr/lib/gcc/x86_64-linux-gnu/9/include/stdint.h \
  /usr/include/stdint.h \
  /usr/include/x86_64-linux-gnu/bits/libc-header-start.h \
  /usr/include/features.h \
  /usr/include/x86_64-linux-gnu/sys/cdefs.h \
  /usr/include/x86_64-linux-gnu/bits/wordsize.h \
  /usr/include/x86_64-linux-gnu/bits/long-double.h \
  /usr/include/x86_64-linux-gnu/gnu/stubs.h \
  /usr/include/x86_64-linux-gnu/gnu/stubs-64.h \
  /usr/include/x86_64-linux-gnu/bits/types.h \
  /usr/include/x86_64-linux-gnu/bits/timesize.h \
  /usr/include/x86_64-linux-gnu/bits/typesizes.h \
  /usr/include/x86_64-linux-gnu/bits/time64.h \
  /usr/include/x86_64-linux-gnu/bits/wchar.h \
  /usr/include/x86_64-linux-gnu/bits/stdint-intn.h \
  /usr/include/x86_64-linux-gnu/bits/stdint-uintn.h \
  /usr/include/x86_64-linux-gnu/asm/types.h \
  /usr/include/asm-generic/types.h \
  /usr/include/asm-generic/int-ll64.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/arch/x86/include/uapi/asm/bitsperlong.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/asm-generic/bitsperlong.h \
  /build/linux-rOxhKL/linux-5.4.0/tools/include/uapi/asm-generic/bitsperlong.h \
  /usr/include/x86_64-linux-gnu/asm/posix_types.h \
  /usr/include/x86_64-linux-gnu/asm/posix_types_64.h \
  /usr/include/asm-generic/posix_types.h \

/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/libctype.o: $(deps_/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/libctype.o)

$(deps_/build/linux-rOxhKL/linux-5.4.0/debian/linux-headers-5.4.0-216-generic/usr/src/linux-headers-5.4.0-216-generic/tools/objtool/libctype.o):
