/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
// Copyright (C) 2005-2017 Andes Technology Corporation

#ifndef __ASM_NDS32_PARAM_H
#define __ASM_NDS32_PARAM_H

#define EXEC_PAGESIZE	8192

#include <asm-generic/param.h>

#endif /* __ASM_NDS32_PARAM_H */
