cmd_arch/x86/include/generated/asm/unistd_32_ia32.h := sh '/build/linux-rOxhKL/linux-5.4.0/arch/x86/entry/syscalls/syscallhdr.sh' '/build/linux-rOxhKL/linux-5.4.0/arch/x86/entry/syscalls/syscall_32.tbl' 'arch/x86/include/generated/asm/unistd_32_ia32.h' 'i386' 'ia32_' ''
