/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
/* Copyright (C) 2005-2019 Andes Technology Corporation */
#ifndef	_FP_UDF_IEX_CRTL_H
#define	_FP_UDF_IEX_CRTL_H

/*
 * The cmd list of sys_fp_udfiex_crtl()
 */
/* Disable UDF or IEX trap based on the content of parameter act */
#define DISABLE_UDF_IEX_TRAP	0
/* Enable UDF or IEX trap based on the content of parameter act */
#define ENABLE_UDF_IEX_TRAP	1
/* Get current status of UDF and IEX trap */
#define GET_UDF_IEX_TRAP	2

#endif /* _FP_UDF_IEX_CRTL_H */
