/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Qualcomm SDM845 interconnect IDs
 *
 * Copyright (c) 2018, Linaro Ltd.
 * Author: Georgi Djakov <georgi.djakov@linaro.org>
 */

#ifndef __DT_BINDINGS_INTERCONNECT_QCOM_SDM845_H
#define __DT_BINDINGS_INTERCONNECT_QCOM_SDM845_H

#define MASTER_A1NOC_CFG		0
#define MASTER_BLSP_1			1
#define MASTER_TSIF			2
#define MASTER_SDCC_2			3
#define MASTER_SDCC_4			4
#define MASTER_UFS_CARD			5
#define MASTER_UFS_MEM			6
#define MASTER_PCIE_0			7
#define MASTER_A2NOC_CFG		8
#define MASTER_QDSS_BAM			9
#define MASTER_BLSP_2			10
#define MASTER_CNOC_A2NOC		11
#define MASTER_CRYPTO			12
#define MASTER_IPA			13
#define MASTER_PCIE_1			14
#define MASTER_QDSS_ETR			15
#define MASTER_USB3_0			16
#define MASTER_USB3_1			17
#define MASTER_CAMNOC_HF0_UNCOMP	18
#define MASTER_CAMNOC_HF1_UNCOMP	19
#define MASTER_CAMNOC_SF_UNCOMP		20
#define MASTER_SPDM			21
#define MASTER_TIC			22
#define MASTER_SNOC_CNOC		23
#define MASTER_QDSS_DAP			24
#define MASTER_CNOC_DC_NOC		25
#define MASTER_APPSS_PROC		26
#define MASTER_GNOC_CFG			27
#define MASTER_LLCC			28
#define MASTER_TCU_0			29
#define MASTER_MEM_NOC_CFG		30
#define MASTER_GNOC_MEM_NOC		31
#define MASTER_MNOC_HF_MEM_NOC		32
#define MASTER_MNOC_SF_MEM_NOC		33
#define MASTER_SNOC_GC_MEM_NOC		34
#define MASTER_SNOC_SF_MEM_NOC		35
#define MASTER_GFX3D			36
#define MASTER_CNOC_MNOC_CFG		37
#define MASTER_CAMNOC_HF0		38
#define MASTER_CAMNOC_HF1		39
#define MASTER_CAMNOC_SF		40
#define MASTER_MDP0			41
#define MASTER_MDP1			42
#define MASTER_ROTATOR			43
#define MASTER_VIDEO_P0			44
#define MASTER_VIDEO_P1			45
#define MASTER_VIDEO_PROC		46
#define MASTER_SNOC_CFG			47
#define MASTER_A1NOC_SNOC		48
#define MASTER_A2NOC_SNOC		49
#define MASTER_GNOC_SNOC		50
#define MASTER_MEM_NOC_SNOC		51
#define MASTER_ANOC_PCIE_SNOC		52
#define MASTER_PIMEM			53
#define MASTER_GIC			54
#define SLAVE_A1NOC_SNOC		55
#define SLAVE_SERVICE_A1NOC		56
#define SLAVE_ANOC_PCIE_A1NOC_SNOC	57
#define SLAVE_A2NOC_SNOC		58
#define SLAVE_ANOC_PCIE_SNOC		59
#define SLAVE_SERVICE_A2NOC		60
#define SLAVE_CAMNOC_UNCOMP		61
#define SLAVE_A1NOC_CFG			62
#define SLAVE_A2NOC_CFG			63
#define SLAVE_AOP			64
#define SLAVE_AOSS			65
#define SLAVE_CAMERA_CFG		66
#define SLAVE_CLK_CTL			67
#define SLAVE_CDSP_CFG			68
#define SLAVE_RBCPR_CX_CFG		69
#define SLAVE_CRYPTO_0_CFG		70
#define SLAVE_DCC_CFG			71
#define SLAVE_CNOC_DDRSS		72
#define SLAVE_DISPLAY_CFG		73
#define SLAVE_GLM			74
#define SLAVE_GFX3D_CFG			75
#define SLAVE_IMEM_CFG			76
#define SLAVE_IPA_CFG			77
#define SLAVE_CNOC_MNOC_CFG		78
#define SLAVE_PCIE_0_CFG		79
#define SLAVE_PCIE_1_CFG		80
#define SLAVE_PDM			81
#define SLAVE_SOUTH_PHY_CFG		82
#define SLAVE_PIMEM_CFG			83
#define SLAVE_PRNG			84
#define SLAVE_QDSS_CFG			85
#define SLAVE_BLSP_2			86
#define SLAVE_BLSP_1			87
#define SLAVE_SDCC_2			88
#define SLAVE_SDCC_4			89
#define SLAVE_SNOC_CFG			90
#define SLAVE_SPDM_WRAPPER		91
#define SLAVE_SPSS_CFG			92
#define SLAVE_TCSR			93
#define SLAVE_TLMM_NORTH		94
#define SLAVE_TLMM_SOUTH		95
#define SLAVE_TSIF			96
#define SLAVE_UFS_CARD_CFG		97
#define SLAVE_UFS_MEM_CFG		98
#define SLAVE_USB3_0			99
#define SLAVE_USB3_1			100
#define SLAVE_VENUS_CFG			101
#define SLAVE_VSENSE_CTRL_CFG		102
#define SLAVE_CNOC_A2NOC		103
#define SLAVE_SERVICE_CNOC		104
#define SLAVE_LLCC_CFG			105
#define SLAVE_MEM_NOC_CFG		106
#define SLAVE_GNOC_SNOC			107
#define SLAVE_GNOC_MEM_NOC		108
#define SLAVE_SERVICE_GNOC		109
#define SLAVE_EBI1			110
#define SLAVE_MSS_PROC_MS_MPU_CFG	111
#define SLAVE_MEM_NOC_GNOC		112
#define SLAVE_LLCC			113
#define SLAVE_MEM_NOC_SNOC		114
#define SLAVE_SERVICE_MEM_NOC		115
#define SLAVE_MNOC_SF_MEM_NOC		116
#define SLAVE_MNOC_HF_MEM_NOC		117
#define SLAVE_SERVICE_MNOC		118
#define SLAVE_APPSS			119
#define SLAVE_SNOC_CNOC			120
#define SLAVE_SNOC_MEM_NOC_GC		121
#define SLAVE_SNOC_MEM_NOC_SF		122
#define SLAVE_IMEM			123
#define SLAVE_PCIE_0			124
#define SLAVE_PCIE_1			125
#define SLAVE_PIMEM			126
#define SLAVE_SERVICE_SNOC		127
#define SLAVE_QDSS_STM			128
#define SLAVE_TCU			129

#endif
